-- |
-- Module	: NMUTool.Buildd
-- Copyright	: 2009 Kari Pahula <kaol@debian.org>
-- License	: GPLv2 or later

module NMUTool.Buildd where

import Text.ParserCombinators.Parsec
import System.Process
import System.IO (stderr,hPutStrLn)
import Data.List(nub,intersperse)
import Data.Maybe(fromJust)
import NMUTool.Types
import NMUTool.Version

nmuOut :: [Source] -> Name -> [(String,[String])] -> IO ()
nmuOut srcs depcheck nmus = do
  mapM_ (\(pkg,archs) -> if null archs then return ()
                         else putStrLn $ "nmu " ++ pkg ++ '_':((\((Version v),_,_) -> v) $ fromJust $ lookup pkg srcs) ++
                                  " . " ++ (concat $ intersperse " " archs) ++ " . -m'Rebuild against new "++depcheck++"'"
        ) nmus

needNMUs :: [Source] -> IO [(String,[String])]
needNMUs srcs = do
  let query = concat $ intersperse "," $ nub $ map fst srcs
  putStrLn $ "query " ++ (show query)
  buildd <- readProcess "wget" ["-q", "-O", "-", "https://buildd.debian.org/~luk/status/package.php?p=" ++ query] []
--  buildd <- readFile "buildd.out"
  case parse statusParser "" buildd of
    Right x -> return x
    Left x -> do hPutStrLn stderr "error parsing reply from https://buildd.debian.org/~luk/status/"
                 hPutStrLn stderr $ show x
                 return []

statusParser :: Parser [(String,[String])]
statusParser = do
  manyTill anyChar $ try $ string "<th>Package</th>"
  archs <- manyTill (do string "<th><a href=\""
                        skipMany $ noneOf ">"
                        char '>'
                        arch <- many1 $ noneOf "<"
                        manyTill anyChar $ try $ string "</th>"
                        spaces
                        return arch) $ try $ string "</tr>"
  many1 $ do
    try $ string "<tr><td><a"
    skipMany $ noneOf ">"
    char '>'
    pkg <- many1 $ noneOf "<"
    string "</a></td>"
    r <- fmap (foldr (\(a,ins) -> if ins then (a:) else id) [] . zip archs) $ many1 $ do
                 try $ string "<td"
                 spaces
                 installed <- ((try (string "class=installed") >> return True) <|> (skipMany (noneOf ">") >> return False))
                 manyTill anyChar $ try $ string "</td>"
                 spaces
                 return installed
    string "</tr>"
    return $ (\x -> (pkg,x)) r
