-- |
-- Module	: NMUTool.Depwait
-- Copyright	: 2009 Kari Pahula <kaol@debian.org>
-- License	: GPLv2 or later

module NMUTool.Depwait where

import NMUTool.Version
import NMUTool.Types
import qualified Data.Map as Map
import Data.List (nub,intersperse,intersect)
import Data.Maybe (catMaybes,fromMaybe,fromJust)
import Debug.Trace

dwLine :: Eq arch => (name,[([arch],version)]) -> [([arch],[(name,version)])] -> [([arch],[(name,version)])]
dwLine (bin,av) [] = map (\(a,v) -> (a,[(bin,v)])) av
dwLine bav@(bin,av) lx
    | not $ any (`elem` dwas) binas = lx
    | all (`elem` dwas) binas =
        foldr (\(a,nv) -> fromMaybe id $ fmap (\v -> ((a,(bin,v):nv):)) $ lookup a av) [] lx
    | all (\a -> any (a `subsetEq`) dwas) binas =
        dwLine bav $ concatMap (\(a,nv) -> map (\a' -> (a',nv)) $ filter (`subsetEq` a) binas) lx
    | otherwise = dwLine (bin,partav) lx
    where binas = map fst av
          dwas = map fst lx
          partav = concatMap (\(a,v) -> map (\a' -> (a',v)) $ filter (/= []) $ map (intersect a) dwas) av

subsetEq :: (Eq a) => [a] -> [a] -> Bool
subsetEq a b = all (`elem` b) a

dwOut :: (Map.Map Name [([Arch],Version)]) -> [Source] -> IO ()
dwOut vers = mapM_ dwOut'
    where dwOut' (src,(ver@(Version vstr),bs,_)) = do
            let deps = catMaybes $ map (\b -> fmap (\x -> (b,x)) $ Map.lookup b vers) (nub $ concatMap devdeps bs)
--            let deps = map (\b -> (\x -> (b,x)) $ fromJust $ Map.lookup b vers) (nub $ concatMap devdeps bs)
            let line = foldr dwLine [] deps
            let pref = "dw " ++ src ++ '_':vstr ++ " . "
            mapM_ (\(a,nv) -> putStr $
                   pref ++ (concat (intersperse " " a)) ++ " . -m'" ++
                   (concat (intersperse ", " (map (\(n,(Version v)) -> n ++ " (<< " ++ v ++ ")") nv))) ++ "'\n") line
