-- |
-- Module	: NMUTool.Madison
-- Copyright	: 2009 Kari Pahula <kaol@debian.org>
-- License	: GPLv2 or later

module NMUTool.Madison(callMadison) where

import Text.ParserCombinators.Parsec
import System.Process
import Data.List (nub)
import qualified Data.Map as Map
import NMUTool.Version
import NMUTool.Types
import System.Exit
import Debug.Trace

{-
data ArchVersions = ArchVersions { binary :: String
                                 , versions :: [(Version, [String])]
                                 }
                  deriving (Show, Eq)
-}

madisonparser :: Parser (Name,[Arch],Version)
madisonparser = do
  let sepSpace = skipMany space >> char '|' >> skipMany space
  skipMany space
  bin <- manyTill anyChar $ char ' '
  sepSpace
  ver <- fmap Version $ manyTill anyChar $ char ' '
  sepSpace
  string "unstable"
  sepSpace
  archs <- sepBy1 (many1 $ alphaNum <|> char '-') (skipMany1 $ space <|> char ',')
  return (bin,filter (/= "source") archs,ver)

callMadison :: [Source] -> IO (Map.Map Name [([Arch],Version)])
callMadison pkgs = do
  let madison = (["-s", "unstable"] ++ (nub $ concatMap sbins pkgs))
  putStrLn $ "pkgs " ++ (show $ map fst pkgs)
  putStrLn $ "madison " ++ (show madison)
  fmap ((foldr (\(b,a,v) -> Map.insertWith (++) b [(a,v)]) Map.empty) .
        (map (\x -> case x of Right z -> z
                              Left z -> error (show z))) .
        (map (parse madisonparser "")) .
        lines) $ readProcess "rmadison" madison []

--
-- $ readFile "rmadison.out"
