-- |
-- Module	: NMUTool.Version
-- Copyright	: 2009 Kari Pahula <kaol@debian.org>
-- License	: GPLv2 or later

module NMUTool.Version(Version(..),sourceVersion) where

import Data.Char (isDigit)
import Control.Monad (guard)
import Data.Maybe (fromMaybe)

newtype Version = Version String deriving (Show, Eq)

sourceVersion :: String -> Version
sourceVersion s = Version $ fromMaybe s $ do
                    let (n,a) = span isDigit $ reverse s
                    guard $ n /= ""
                    ('b':'+':r) <- Just a
                    return $ reverse r

instance Ord Version where
    compare a b =
        let (e1, v1) = epochVer a
            (e2, v2) = epochVer b in
        case compare e1 e2 of
          LT -> LT ; GT -> GT
          EQ -> compareVersion a b

epochVer (Version v) = case span isDigit v of
                         (e, ':':r) -> (read e, r)
                         _ -> (0::Int, v)

compareVersion (Version "") (Version "") = EQ
compareVersion (Version a) (Version b) =
    let (s1,r1) = span (not . isDigit) a
        (s2,r2) = span (not . isDigit) b in
    case compareStringPart s1 s2 of
      LT -> LT ; GT -> GT
      EQ -> let (v1,r1') = parseNumberPart r1
                (v2,r2') = parseNumberPart r2 in
            case compare v1 v2 of
              LT -> LT ; GT -> GT
              EQ -> compareVersion (Version r1') (Version r2')

compareStringPart ('~':a) ('~':b) = compareStringPart a b
compareStringPart ('~':_) _ = LT
compareStringPart _ ('~':_) = GT
compareStringPart a b = compare a b

parseNumberPart :: String -> (Int, String)
parseNumberPart n = let (s,r) = span isDigit n in
                    case reads s of
                      [(v,"")] -> (v,r)
                      _ -> (0,"")
